# -*- coding: utf-8 -*-

"""
Basic functionality to work with place data.
"""

from util import load_csv


def from_csv(place_csv, placerelation_csv=None, placerelation_related_places_csv=None):
    """
    Loads place data from the given CSV files.

    Args:
        place_csv (str):
            name of the file containing place data
        placerelation_csv (str, optional)
            name of the file containing placerelation data
        placerelation_related_places_csv (str, optional)
            name of the file containing placerelation_related_places data

    Returns:
        pandas.DataFrame
    """
    places = load_csv(place_csv)
    if placerelation_csv and placerelation_related_places_csv:
        placerelations_from_csv(places, placerelation_csv, placerelation_related_places_csv)
    return places


def placerelations_from_csv(places, placerelation_csv, placerelation_related_places_csv):
    """
    Loads placerelation data to the given places data frame.
    WARNING: The function modifies the given dataframe!

    Args:
        places (pandas.DataFrame):
            data frame containing place data
        placerelation_csv (str)
            name of the file containing placerelation data
        placerelation_related_places_csv (str)
            name of the file containing placerelation_related_places data

    Returns:
        pandas.DataFrame
    """

    TYPES = {
        1: 'is_on_map',
        2: 'is_submap',
        3: 'land_borders',
        4: 'too_small_on_map'
    }

    placerelations = load_csv(placerelation_csv)
    related_places = load_csv(placerelation_related_places_csv)
    related_places_dict = {}
    placerelations_dict = {}
    for i, row in placerelations.iterrows():
        placerelations_dict[row['id']] = (row['place'], TYPES[row['type']])
    for i, row in related_places.iterrows():
        if row['place'] not in related_places_dict:
            related_places_dict[row['place']] = []
        related_places_dict[row['place']].append(placerelations_dict[row['placerelation']])
    places['relations'] = places['id'].map(lambda id: related_places_dict.get(id, []))
    return places



# -*- coding: utf-8 -*-
import unittest
import proso.geography.places as places
import os


class TestBasics(unittest.TestCase):

    def test_from_csv(self):
        data = places.from_csv(
            place_csv=os.environ['RESOURCES_DIR'] + '/place.csv',
            placerelation_csv=os.environ['RESOURCES_DIR'] + '/placerelation.csv',
            placerelation_related_places_csv=os.environ['RESOURCES_DIR'] + '/placerelation_related_places.csv',
            )
        self.assertEqual(len(data), 3)
        relations = sorted(list(data.head().relations)[0])
        self.assertEquals(relations, [(225, 'is_on_map'), (230, 'too_small_on_map')])



# -*- coding: utf-8 -*-

"""
Basic functionality to work with answer data.
"""

from util import load_csv


def from_csv(answer_csv, answer_options_csv=None, answer_ab_values_csv=None, ab_value_csv=None):
    """
    Loads answer data from the given CSV files.

    Args:
        answer_csv (str):
            name of the file containing  answer data
        answer_options_csv (str, optional):
            name of the file containing answer_options data
        answer_ab_values_csv (str, optional):
            name of the file containing answer_ab_values data
        ab_value_csv (str):
            name of the file containing ab_value data

    Returns:
        pandas.DataFrame
    """
    answers = load_csv(answer_csv)
    if answer_options_csv:
        options_from_csv(answers, answer_options_csv)
    if ab_value_csv and answer_ab_values_csv:
        ab_values_from_csv(answers, ab_value_csv, answer_ab_values_csv)
    return answers


def ab_values_from_csv(answers, ab_value_csv, answer_ab_values_csv):
    """
    Loads A/B values to the answers data frame.
    WARNING: The function modifies the given dataframe!

    Args:
        answers (pandas.DataFrame):
            dataframe containing answer data
        ab_value_csv (str):
            name of the file containing ab_value data
        answer_ab_values_csv (str, optional):
            name of the file containing answer_ab_values data

    Returns:
        pandas.DataFrame
    """
    ab_values = load_csv(ab_value_csv)
    answer_ab_values = load_csv(answer_ab_values_csv)
    ab_values_dict = {}
    answer_ab_values_dict = {}
    for i, row in ab_values.iterrows():
        ab_values_dict[row['id']] = row['value']
    for i, row in answer_ab_values.iterrows():
        if row['answer'] not in answer_ab_values_dict:
            answer_ab_values_dict[row['answer']] = []
        answer_ab_values_dict[row['answer']].append(ab_values_dict[row['value']])
    answers['ab_values'] = answers['id'].map(lambda id: answer_ab_values_dict.get(id, []))
    return answers


def options_from_csv(answers, answer_options_csv):
    """
    Loads options to the answers data frame.
    WARNING: The function modifies the given dataframe!

    Args:
        answers (pandas.DataFrame):
            data frame containing answer data
        answer_options_csv (str):
            name of the file containing answer_options data

    Returns:
        pandas.DataFrame
    """
    options = load_csv(answer_options_csv)
    options_dict = {}
    for i, row in options.iterrows():
        if row['answer'] not in options_dict:
            options_dict[row['answer']] = []
        options_dict[row['answer']].append(row['place'])
    answers['options'] = answers['id'].map(lambda id: options_dict.get(id, []))
    return answers



# -*- coding: utf-8 -*-
from datetime import datetime
from pandas import DataFrame


def load_csv(csv_file):
    data = DataFrame.from_csv(csv_file, index_col=False)
    for column in data.columns:
        if column == 'inserted':
            data['inserted'] = data['inserted'].apply(convert_time)
        elif column == 'id':
            data.sort(['id'], inplace=True, ascending=True)
        elif is_list_column(data[column]):
            data[column] = data[column].apply(lambda x: str2list(x, int))
    return data


def convert_time(value):
    return datetime.strptime(value, "%Y-%m-%d %H:%M:%S")


def str2list(x, convert_item=None):
    s = x.strip('[]').replace(' ', '').split(',')
    if convert_item:
        s = map(convert_item, s)
    return s


def is_list_column(column):
    if len(column) == 0:
        return False
    reps = column.head(min(10, len(column)))
    str_type = type('')
    return reps.apply(lambda x: type(x) == str_type and x.startswith('[') and x.endswith(']')).all()



# -*- coding: utf-8 -*-
from setuptools import setup


setup(
    name='proso-geography-data',
    version='0.9.7',
    description='Libs for manipulation with data from PROSO Geography project',
    author='Jan Papousek',
    author_email='jan.papousek@gmail.com',
    packages=['proso.geography'],
    license='Gnu GPL v3',
    url='https://github.com/proso/geography-data-libs/',
    install_requires=[
        'pandas>=0.13'
    ],
)



# -*- coding: utf-8 -*-
import unittest
import proso.geography.answers as answers
import os


class TestBasics(unittest.TestCase):

    def test_from_csv(self):
        data = answers.from_csv(
            answer_csv=os.environ['RESOURCES_DIR'] + '/answer.csv',
            answer_options_csv=os.environ['RESOURCES_DIR'] + '/answer_options.csv',
            answer_ab_values_csv=os.environ['RESOURCES_DIR'] + '/answer_ab_values.csv',
            ab_value_csv=os.environ['RESOURCES_DIR'] + '/ab_value.csv'
            )
        self.assertEqual(len(data), 2)
        options = sorted(list(data.head().options)[0])
        self.assertEqual(options, [99, 101, 106, 177])
        ab_values = sorted(list(data.head().ab_values)[0])
        self.assertEqual(ab_values, ['recommendation_by_additive_function'])
